Simfunction [T,Y] = Sim_twoAtoBr2(varargin)
% Usage: Sim_twoAtoBr2(kf,kr,A0,B0,deltaA,deltaB,Vratio,t_end_set,saveflag)
%  as in >> Sim_twoAtoBr2(3e-2,6e-2,1,4,5,5,2,0,0);  % defaults
% Function to illustrate LeChatelier's principle and dynamic equilibrium by
%    simulating the gas phase dimerization reaction 2 A(g) <-> B(g)
%    as a function of time, with a change in volume, change in P(A), change
%    in P(B), and return to original volume to illustrate induced changes
%    in the rxn quotient Q and its reversion to Kp. 
% Jason Kahn U Maryland 11/8/2016  version 1.3
%   Change log -- 1.3 cosmetics,  checked capitalization of function names
%                 1.2 (r2) calculation of relaxation times to make plots
%                 look good
%                 1.01 first version provided
% Returns a time vector and concentration matrix, although not sure what
%    one would do with the returned values.
% Integrates differential rate law, plots pressures, rates, and Q vs. K
% Arguments (all optional) must be in the following order:
% kf,kr,A0,B0,deltaA,deltaB,volume change ratio (compression for >1),
%    ....set relaxation time (0 for leave alone),savefig flag
% You can leave arguments off at the end but no skipping.
% With no arguments defaults to: Sim_twoAtoBr2(3e-2,6e-2,1,4,5,5,2,0,0);
% Calls function TwoAtoB (provided separately) to calculate differential
%   rates that are then integrated using ode45
% Calls function A2Brates (provided separately) to regenerate rates for
%    plotting purposes
% Calls function export_fig if and only if savefig flag is set to 1
%   Download export_fig from http://www.mathworks.com/matlabcentral/fileexchange/23629-exportfig
%
% Define rate constants and initial partial pressures
% Defaults:
kf = 3e-2;  % now that we calc. relaxation time can change k's freely
kr = 6e-2;
A0 = 1;
B0 = 4;
% Amounts of A and B to be added or removed (magically...) during the time course
% If we hit a negative partial pressure it is just set to zero without
% consultation
deltaA = 5;
deltaB = 5;
% Compression and expansion factors
% Vratio1 is done first. Specified as fold-compression, so compression is >1, expansion is <1.
Vratio1 = 2;  % If it's less than 1 it is really an expansion
% Returns to original volume later
% Default is not to write files. Set savefig = 0 to prevent saving
% Set savefig to 1 to activate saving images
tfactor = 2; % used to set scaling of time axis
t_end_set = 0;
savefig = 0;
optargs = {kf kr A0 B0 deltaA deltaB Vratio1 t_end_set savefig};
numopts = 9;
% Change default values if user provides them
numvarargs = length(varargin);
if numvarargs > numopts
    error('Sim_twoAtoBr2:TooManyInputs', ...
        'Sim_twoAtoBr2 allows at most %d optional inputs',numopts);
end

% now put these defaults into the valuesToUse cell array, 
% and overwrite the ones specified in varargin.
optargs(1:numvarargs) = varargin;

% Place optional args back into in memorable variable names
[kf,kr,A0,B0,deltaA,deltaB,Vratio1,t_end_set,savefig] = optargs{:};
if (A0 < 0 || B0 < 0)
    error('Negative initial [A] or [B] detected. We must think positively.')
end
% Estimate relaxation time, allow for resetting total time
ctr = sqrt(1/(kr^2 + 8*kf*kr*(max(A0,A0+deltaA) + 2*max(B0,B0+deltaB))));
if t_end_set == 0
    tr = ctr;
else
    tr = t_end_set/(8*tfactor);
end
disp(['kf = ',num2str(kf)])
disp(['kr = ',num2str(kr)])
disp(['A at t0 = ',num2str(A0)])
disp(['B at t0 = ',num2str(B0)])
disp(['delta A = ',num2str(deltaA)])
disp(['delta B = ',num2str(deltaB)])
disp(['Compression Ratio = ',num2str(Vratio1)])
disp(['Calc. relaxation time is ',num2str(ctr)])
if t_end_set ~= 0
disp(['Set total time is ',num2str(t_end_set)])
else
disp(['Calc. total time for display is ',num2str(tr*8*tfactor)])    
end
disp(['Save figure flag is ',num2str(savefig)])
Vratio2 = 1/Vratio1;  % Returns to original volume at t4

% pnum is for numbering plots sequentially. 
% Limit number of files exported to lim_files
% savefig = 1 means we are saving files
if savefig == 1;
    lim_files = 50;
    fileout = 'Rxn_plot_TwoAtoB_';
    pnum = 1;
    while (pnum <= lim_files)
        fntest = [fileout,num2str(pnum),'.jpg'];
        if (exist(fntest,'file'))
            pnum = pnum+1;
        else
            disp(['Exported file is ',fntest])
            break
        end
    end
    if (pnum == lim_files+1)
        disp(['Exceeded file count: overwriting ',fntest])
    end
end

% Times for compression, add/remove A, add/remove C, expansion, and end
% Set to coordinate with labeling, legends, etc.
t1_end = 1.8*tr*tfactor;
t2_end = 3.0*tr*tfactor;
t3_end = 4.2*tr*tfactor;
t4_end = 5.4*tr*tfactor;
t5_end = 8.0*tr*tfactor;   % which equals tr or t_end_set
% Change Qscale and Rscale to optimize appearance; should not be necessary
Qscale = 4.0; % was 3.5
Rscale = 2.0;
%---------------------------
% Should not have to touch anything below here

Keq = kf/kr;
% Integrate differential equation to give initial time course. This is the
% point of the whole exercise. We pass TwoAtoB the time, but in fact this
% is unnecessary. 
options = odeset('RelTol',1e-4,'AbsTol',[1e-6 1e-6]);
[T1,Y1] = ode45(@(t,concs) TwoAtoB(t,concs,kf,kr),[0 t1_end],[A0 B0]',options);
lastconcs = Y1(end,:);

% Instantaneous compression (or expansion, depending on Vratio1) 
%     at t1_end and allow to evolve until t2_end
[T2,Y2] = ode45(@(t,concs) TwoAtoB(t,concs,kf,kr),[t1_end t2_end],Vratio1*lastconcs,options);
lastconcs = Y2(end,:);

% Add (or remove) A at t2_end, evolve to t3_end
newA = max(0,lastconcs(1)+deltaA);
[T3,Y3] = ode45(@(t,concs) TwoAtoB(t,concs,kf,kr),[t2_end t3_end],[newA lastconcs(2)],options);
lastconcs = Y3(end,:);

% Add (or remove) B at t3_end, evolve to t4_end
newB = max(0,lastconcs(2)+deltaB);
[T4,Y4] = ode45(@(t,concs) TwoAtoB(t,concs,kf,kr),[t3_end t4_end],[lastconcs(1) newB],options);
lastconcs = Y4(end,:);

% Instantaneous compression (or expansion) at t4_end and allow to evolve until t5_end
[T5,Y5] = ode45(@(t,concs) TwoAtoB(t,concs,kf,kr),[t4_end t5_end],Vratio2*lastconcs,options);
%lastconcs = Y5(end,:);

% Collect time and concentration arrays

T = [T1 ; T2 ; T3 ; T4 ; T5];
Y = [Y1 ; Y2 ; Y3 ; Y4 ; Y5];
%
if (min(min(Y)) < 0)
    disp('WARNING:')
    disp('    Negative partial pressure detected: results are likely WRONG!   ')
end
% recalculate instantaneous rates. We do not pass A2Brates the time.
numt = length(T);
rates = zeros(numt,2);
for i=1:numt
%    Y(i,:);
    rates(i,:) = A2Brates(Y(i,:),kf,kr);
end

% Plot results
set(0,'defaultAxesFontName', 'Arial')
set(0,'defaultTextFontName', 'Arial')
set(0,'defaultAxesFontSize', 14)
set(0,'defaultAxesFontWeight', 'bold')

%scrsz = get(0,'ScreenSize');
figure('Position',[10 10 1024 768])
hold on
sub1 = subplot(3,1,1);
plot(T,Y(:,1),'-r',T,Y(:,2),'-.b',T,Y(:,1)+Y(:,2),'c','LineWidth',2)
xlim(sub1,[0 t5_end]);
leg1 = legend('P(A)','P(B)','Total P');
set(leg1,'FontSize',16);
%get(leg1,'FontName')
title('Partial Pressures of A and B','FontSize',16,'FontWeight','bold')
ylabel('P(A) and P(B)','FontSize',14,'FontWeight','bold')
text1 = text(min(T)+0.03*(max(T)-min(T)),max(Y(:,1)+Y(:,2))*0.9,'Rxn: 2 A <-> B','FontSize',14,'FontWeight','bold');
set(text1,'FontName','Arial')
set(text1,'FontSize',18) 
sub2 = subplot(3,1,2);
plot(T,(rates(:,1)),'-g',T,(rates(:,2)),'-.k','LineWidth',2)
% ylim should be adaptive depending on rate constants and concentrations
% to show the rates to best advantage
xlim(sub2,[0 t5_end]);
ylim(sub2,[0 max(median(rates(:,1)),Rscale*max(rates(:,2)))]);
ylabel('Rates','FontSize',14,'FontWeight','bold')
title('Rates of Forward and Reverse Reactions Converge','FontSize',14,'FontWeight','bold')
leg2 = legend('Rate of forming B','Rate of going back to A');
set(leg2,'FontSize',16);
% Plot Q vs K
sub3 = subplot(3,1,3);
plot(T,Y(:,2)./(Y(:,1).^2),'LineWidth',2)
maxQ = max(Y(:,2)./(Y(:,1).^2));
hold on
plot(T,(kf/kr),'ok')
xlim(sub3,[0 t5_end]);
ylim(sub3,[0 max(1.3*Keq,min(maxQ,Qscale*Keq))])
ymax = ylim(sub3);
leg3 = legend('Q',['kf = ',num2str(kf)],['kr = ',num2str(kr)],['Value of Keq = ',num2str(Keq)],...
    ['A at t0 = ',num2str(A0)],['B at t0 = ',num2str(B0)],['delta A = ',num2str(deltaA)],...
    ['delta B = ',num2str(deltaB)],['Compression Ratio = ',num2str(Vratio1)],'Location','SouthEast');
set(leg3,'FontSize',11);
title('Reaction Quotient Q Approaches K','FontSize',14,'FontWeight','bold')
ylabel('Rxn Quotient Q','FontSize',14,'FontWeight','bold')
xlabel('Time','FontSize',14,'FontWeight','bold')
if (Vratio1 > 1)
text(t1_end,0.9*ymax(2),'Compress','HorizontalAlignment','center','BackgroundColor','w','FontSize',14,'FontWeight','bold')
else
text(t1_end,0.9*ymax(2),'Expand','HorizontalAlignment','center','BackgroundColor','w','FontSize',14,'FontWeight','bold')    
end
if (deltaA >= 0)
text(t2_end,0.9*ymax(2),'Add A','HorizontalAlignment','center','BackgroundColor','w','FontSize',14,'FontWeight','bold')
else
text(t2_end,0.9*ymax(2),'Remove A','HorizontalAlignment','center','BackgroundColor','w','FontSize',14,'FontWeight','bold')
end
if (deltaB >= 0)
text(t3_end,0.9*ymax(2),'Add B','HorizontalAlignment','center','BackgroundColor','w','FontSize',14,'FontWeight','bold')
else
text(t3_end,0.9*ymax(2),'Remove B','HorizontalAlignment','center','BackgroundColor','w','FontSize',14,'FontWeight','bold')
end
if (Vratio1 > 1)
text(t4_end,0.9*ymax(2),'Re-Expand','HorizontalAlignment','center','BackgroundColor','w','FontSize',14,'FontWeight','bold')
else
text(t4_end,0.9*ymax(2),'Re-Compress','HorizontalAlignment','center','BackgroundColor','w','FontSize',14,'FontWeight','bold')
end
% Don't know what the code below was doing -- resize text if exporting?
%figureHandle = gcf;
%set(findall(figureHandle,'type','text'),'fontSize',14,'fontWeight','bold','fontname','Arial')
%set(leg3,'FontSize',11);
% Export jpeg
if (savefig == 1)
    export_fig (fntest,'-jpeg')
end
 


